<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\File;

class GenerateApiVersionCommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'generate:version {version} {old_version?}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'This command to generate all classes with different version';

    protected array $folders = [
        'app'.DIRECTORY_SEPARATOR.'Http'.DIRECTORY_SEPARATOR.'Controllers'.DIRECTORY_SEPARATOR.'V',
        'app'.DIRECTORY_SEPARATOR.'Http'.DIRECTORY_SEPARATOR.'Requests'.DIRECTORY_SEPARATOR.'V',
        'app'.DIRECTORY_SEPARATOR.'Http'.DIRECTORY_SEPARATOR.'Services'.DIRECTORY_SEPARATOR.'V',
        'app'.DIRECTORY_SEPARATOR.'Http'.DIRECTORY_SEPARATOR.'Resources'.DIRECTORY_SEPARATOR.'V',
    ];

    protected int $version;
    protected int $old_version;

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        if (! $this->checkFoldersExists()) {
            return false;
        }

        $this->copyDirs();
        $this->copyApiRoute();
    }

    protected function checkFoldersExists(): bool
    {
        $this->setVersions();

        if ( $this->version == $this->old_version ) {
            $this->error("Please Select Different Versions");
            return false;
        }

        $check = true;
        foreach ($this->folders as $path) {
            $old_path = $path.$this->old_version;
            if ( ! file_exists( $old_path ) ) {
                $this->line("This dir <options=bold>$old_path</> does not exists");
                $check = false;
            }
        }

        return $check;
    }

    protected function copyDirs()
    {
        foreach ($this->folders as $path) {
            File::copyDirectory($path.$this->old_version , $path.$this->version);
            $this->updateFilesVersionNamespace( $path.$this->version );
        }
    }

    protected function copyApiRoute()
    {
        $path = base_path('routes'.DIRECTORY_SEPARATOR."api-v{VERSION}.php");
        $content = file_get_contents( str_replace("{VERSION}", $this->old_version, $path) );
        File::put(str_replace("{VERSION}", $this->version, $path), $content);
        $this->line("Please copy the following code and append it in <options=bold>app\Providers\RouteServiceProvider.php</>");
        $this->comment('Route::prefix(\'rep/api/v'.$this->version.'\')
                        ->middleware(\'json.response\')
                        ->namespace("$this->namespace\\V'.$this->version.'\\Rep")
                        ->group(base_path(\'routes/api-v'.$this->version.'.php\'));');
    }

    protected function updateFilesVersionNamespace($dir)
    {
        foreach (File::allFiles($dir) as $file) {
            $content = $this->replaceVersionNumber( file_get_contents($file->getPathname()) );
            File::put($file->getPathname(), $content);
        }
    }

    protected function replaceVersionNumber($text)
    {
        return str_replace("V{$this->old_version}", "V{$this->version}", $text);
    }

    protected function makeDir($dir)
    {
        if ( ! file_exists( $dir ) ) {
            mkdir($dir, 0777, true);
        }
    }

    protected function setVersions()
    {
        $this->version = $this->old_version = $this->argument('version');
        if ($this->argument('old_version')) {
            $this->old_version = $this->argument('old_version');
            return ;
        }

        do {
            $this->old_version--;
        } while ( !file_exists( $this->folders[0].$this->old_version ) );
    }
}
