<?php

namespace App\Http\Controllers\V3\Rep;

use App\Http\Controllers\V3\GeneralApiController;
use App\Http\Resources\V3\ItemTypesResource;
use App\Models\ItemType;
use App\Http\Requests\V3\ItemTypeRequest;
use Illuminate\Http\Request;

class ItemTypeController extends GeneralApiController
{
    public function __construct()
    {
        parent::__construct(ItemType::class, ItemTypesResource::class);
    }

    public function store(ItemTypeRequest $request)
    {
        $validated = $request->validated();
        $this->uploadImage($request, $validated);
        $row = $this->model::create($validated);
        return $row
            ? $this->sendResponse('Type created successfully', ['data' => new $this->resource($row)])
            : $this->sendError('Error try again');
    }

    public function update(ItemTypeRequest $request, $id)
    {
        $row = $this->model::find($id);
        if (!$row) return $this->sendError('This Type not exists');
        $validated = $request->validated();
        $this->uploadImage($request, $validated);
        $row->update($request->validated());
        return $this->sendResponse('Type updated successfully', ['data' => new $this->resource($row)]);
    }

    protected function uploadImage(Request $request, array &$validated)
    {
        if ($request->has('upload_img')) {
            $validated['upload_img'] = $request->file('upload_img')->hashName();
            $request->file('upload_img')->storeAs('upload', $validated['upload_img']);
        }
    }

    public function query(?int $id = null): \Illuminate\Database\Eloquent\Builder
    {
        return $this->model::query()->select('id', 'name', 'upload_img')->when($id, fn($q) => $q->where('id', $id));
    }
}
