<?php

namespace App\Http\Controllers\V3\Rep;

use App\Http\Controllers\V3\BasicApiController;
use App\Http\Requests\V3\SpendRequest;
use App\Http\Resources\V3\SpendingResource;
use App\Models\{Badrshop, ClientTransaction, SalePoint, Spending, SpendItem};
use Illuminate\Support\Facades\{DB, File};
use Exception;
use PDOException;

class SpendController extends BasicApiController
{
    public function index()
    {
        $check = Badrshop::select('assign_users_to_expenses')->first()->assign_users_to_expenses ?? false;
        $rows = SpendItem::select('id', 'name')->when($check, function($query) {
                                $query->whereHas('users', fn($query) => $query->where('users.id', authId()) );
                            })->get()->toArray();
        return $this->returnData($rows);
    }

    public function store(SpendRequest $request)
    {
        logger( "Start Store Spending | user_id : " . authId());
        logger( json_encode( $request->all() ) );
        try {
            DB::beginTransaction();
                $sale_point = SalePoint::select('id', 'money_point')->whereId(salePointId())->first();

                $validated = array_merge($request->validated(), ['sale_point_id' => $sale_point->id]);
                logger(json_encode($validated));

                $row = $this->createSpend( $validated );

                $validated['money_point'] = $sale_point->money_point - $request->amount;

                $sale_point->decrement('money_point', $validated['amount']);

                $this->createClientTransation($validated);
                logger('done');
            DB::commit();
            return $this->sendResponse('تم الحفظ بنجاح', ['data' => new SpendingResource($row)]);
        } catch (Exception $e) {
            DB::rollBack();

            return $this->handleDuplicate($validated, $e)
                    ? $this->sendResponse('تم الحفظ بنجاح', ['data' => new SpendingResource($row)])
                    : $this->sendError( $e->getMessage() );
        }
    }

    protected function handleDuplicate(array $data, Exception $th): mixed
    {
        logger(json_encode($th));
        $row = null;
        if($th instanceof PDOException) {
            $row = Spending::where([
                    'amount'       => $data['amount'],
                    'spend_id'     => $data['term_id'],
                    'spend_date'   => $data['date'],
                    'spend_type'   => 1,
                    'shop_id'      => shopId(),
                ])->first();
        }
        return $row;
    }

    protected function createSpend(array $validated): Spending
    {
        return Spending::create([
            'spend_id' => $validated['term_id'],
            'spend_date' => $validated['date'],
            'amount' => $validated['amount'],
            'spend_type' => 1,
            'spend_for' => 0,
            'sale_point' => $validated['sale_point_id'],
            'note' => $validated['notes'],
            'bill_add_id' => 0,
            'vat_value' => 0,
            'spend_file' => $this->uploadFile(request()->get('file')),
            'is_confirmed' => 1,
            'bill_no' => 0,
            'local_id' => $validated['local_id']
        ]);
    }

    protected function createClientTransation(array $validated): void
    {
        ClientTransaction::create([
            'date_time' => $validated['date_time'],
            'amount' => $validated['amount'],
            'type' => 10,
            'effect' => 0,
            'pay_day' => $validated['date'],
            'safe_balance' => $validated['money_point'],
            'safe_point_id' => $validated['sale_point_id'],
            'notes' => $validated['notes'],
            'spend_id' => $validated['term_id'],
            'user_id' => authId(),
            'client_id' => authId(),
            'balance' => 0,
            'bill_id' => 0,
            'sale_back_id' => 0,
            'problem_id' => 0,
            'supplier_id' => 0,
            'salary_month' => 0,
            'OutTransactionID' => 0,
        ]);
    }

    protected function uploadFile($file = null): string
    {
        if (! $file) return '';
        $file_name = date("Y-m-d") . '_' . time() . '.' . strtolower($file->getClientOriginalExtension());
        // $name_one = shopId() . '/spending/';
        // $destinationPath = config('app.img_url').$name_one;

        $name_one = 'upload/'.shopId().'/spending/';
        $destinationPath = base_path().DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.config('app.project_name', 'badrshop').DIRECTORY_SEPARATOR.$name_one;

        File::isDirectory($destinationPath) or File::makeDirectory($destinationPath, 0755, true, true);
        $file->move($destinationPath, $file_name);
        return $name_one . $file_name;
    }
}
