<?php

namespace App\Http\Controllers\V4;

use App\Http\Controllers\Controller;
use Illuminate\Http\Resources\Json\JsonResource;

class BasicApiController extends Controller
{
    protected $shop_id;
    protected $store_id;
    protected $sale_point;

    public function __construct()
    {
        $this->shop_id    = shopId();
        $this->sale_point = salePointId();
    }

    public function sendResponse(string $message = '', array $result = []): \Illuminate\Http\JsonResponse
    {
        $response = ['status' => true];

        if($message != '') {
            $response = array_merge($response, ['message' => $message]);
        }

        if(! empty($result)) {
            $response = array_merge($response, $result);
        }

        return response()->json($response, 200);
    }

    public function sendError(string $message = '', $errorMessages = [], $code = 200): \Illuminate\Http\JsonResponse
    {
        $response = ['status' => false];

        if($message != '') {
            $response = array_merge($response, ['message' => $message]);
        }

        if(!empty($errorMessages)) {
            $response['errors'] = $errorMessages;
        }

        return response()->json($response, $code);
    }

    public function returnData(array|JsonResource $rows = []): \Illuminate\Http\JsonResponse
    {
        return count($rows)
            ? $this->sendResponse(result: ['data' => $rows])
            : $this->sendError('No Data Found');
    }
}
