<?php

namespace App\Http\Controllers\V4\Rep;

use App\Exports\ClientBalanceExport;
use App\Http\Controllers\V4\GeneralApiController;
use App\Http\Resources\V4\ClientsResource;
use App\Http\Services\V4\ClientService;
use App\Models\{Badrshop, Client, ClientsGroup};
use Illuminate\Http\Request;
use Maatwebsite\Excel\Facades\Excel;
use Exception;
use Illuminate\Support\Arr;

class ClientController extends GeneralApiController
{
    public function __construct()
    {
        parent::__construct(Client::class, ClientsResource::class);
    }

    public function clientsGroups()
    {
        $groups = ClientsGroup::select('id', 'name', 'group_discount')->when(authUser()->groups()->count(), function ($query) {
                                    $query->where('id', authUser()->groups()->value('group_id'));
                                })->get()->toArray();
        $clients = $this->query()->get();
        return $this->returnData(['groups' => $groups, 'clients' => ClientsResource::collection($clients)]);
    }

    public function store(Request $request)
    {
        $allow_lines = Badrshop::select('allow_lines')->where('serial_id', shopId())->first()?->allow_lines && ! authUser()->line;
        if ($allow_lines) return $this->sendError('لا يمكن اضافة عميل لعدم وجود خط');

        logger( '____________________________________________' );
        logger( 'Rows : ' . json_encode($request->all()) );

        $data = $request->has('clients') ? $request->all() : ['clients' => [ $request->all() ]];
        $rows = (new ClientService())->create($data, $allow_lines);

        $response = $request->has('clients') || count($rows['rows']) == 0 ? ['data' => $rows] : ['client' => new $this->resource($rows['rows'][0])];

        logger( "response : " . json_encode($response ) );
        return $rows['rows'] instanceof Exception || count($rows['rows']) == 0
                ? $this->sendError('خطأ في البيانات', Arr::flatten( $rows['errors'] ))
                : $this->sendResponse('Client created successfully', $response);
    }

    public function update(Request $request, $client)
    {
        logger("___________________Update Client {$client}_________________________");
        logger( 'Rows : ' . json_encode($request->all()) );
        $data = ['clients' => [ $request->all() ]];
        $data['clients'][0]['client_id'] = $data['clients'][0]['id'];
        $rows = (new ClientService())->create($data, false);
        $client_row = count($rows['rows']) ? $rows['rows'][0] : new Client();
        $response = ['data' => new $this->resource($client_row)];
        logger( "response : " . json_encode($response) );
        return $rows['rows'] instanceof Exception || count($rows['rows']) == 0
                ? $this->sendError('خطأ في البيانات', Arr::flatten( $rows['errors'] ))
                : $this->sendResponse('Client updated successfully', $response);
    }

    public function balanceSheetExcel($client_id, Request $request)
    {
        $client = Client::select('id', 'client_name as name', 'shop_id')->find($client_id);

        if (! $client)
            return $this->sendError('This client not found!');

        ini_set('max_execution_time', 0);
        ini_set('max_input_time', 0);
        ini_set('memory_limit', '256M');

        $file_name = $this->checkFile($client_id);
        Excel::store(new ClientBalanceExport($client, $request->all()), $file_name);
        return $this->sendResponse(result: ['file' => url($file_name)]);
    }

    protected function checkFile($folder)
    {
        $path =  "excel".DIRECTORY_SEPARATOR.$folder;
        $file_path = $path . DIRECTORY_SEPARATOR . "client_balance.xlsx";

        if (file_exists( $file_path ))
            unlink( $file_path );

        if (!file_exists($path))
            mkdir($path, 0777, true);

        if (!file_exists($file_path))
            touch(  $file_path, strtotime('-1 days'));

        return $file_path;
    }

    public function query(?int $id = null): \Illuminate\Database\Eloquent\Builder
    {
        $module = Badrshop::select('module')->first()->module ?? null;
        $role = $module == 2 ? [1, 2] : [0, 2];
        return $this->model::query()->selectRaw('id, client_name, tele, client_tax_number, shop_id, address, group_id, FORMAT(balance, 2) as balance, lat, lon, price_list_id')
                            ->where('shop_id', shopId())
                            ->whereIn('client_role', $role)
                            ->when($id, fn($q) => $q->where('id', $id))
                            ->when(authUser()->groups()->count(), function ($query) {
                                $query->where('group_id', authUser()->groups()->pluck('group_id')->toArray());
                            });
    }
}
