<?php

namespace App\Http\Controllers\V4\Rep;

use App\Http\Controllers\V4\BasicApiController;
use App\Http\Resources\V4\{SettingsResource, StoresResource};
use App\Models\{Badrshop, City, PriceList, SalePoint, Store};

class GeneralListController extends BasicApiController
{
    public function settings()
    {
        $setting = Badrshop::select('multi_price', 'activate_maintenance_mode', 'telephone_required', 'generate_qr', 'client_address_required', 'select_client', 'sale_cash', 'tracking', 'make_bill', 'logo_path' ,'company_ratio', 'apply_vats as bill_adds' , 'sale_details', 'allow_lines', 'sale_details_safe')->first();
        return $this->sendResponse(result: ['data' => new SettingsResource($setting)]);
    }

    public function prices()
    {
        $rows = PriceList::select('id', 'list_name')->get()->toArray();
        return $this->returnData($rows);
    }
    public function cities()
    {
        $rows = City::select('id', 'city_name as name')->get()->toArray();
        return $this->returnData($rows);
    }

    public function salePoints()
    {
        $rows = SalePoint::select('id', 'point_name as name', 'point_type')
                            ->when(in_array(authUser()->show_points, [1, 2]), fn($query) => $query->where('id', salePointId()))
                            ->when(authUser()->show_points == 3, fn($query) => $query->where('store_id', storeId()))
                            ->get();
        return $this->returnData([
            'points'  => $rows->where('point_type', 1)->toArray(),
            'network' => $rows->where('point_type', 2)->toArray()
        ]);
    }

    public function stores()
    {
        $rows = Store::query()->select('id', 'store_name', 'img')->get();
        return $this->sendResponse(result: ['data' => StoresResource::collection($rows)]);
    }
}
