<?php

namespace App\Http\Controllers\V4\Rep;

use App\Http\Controllers\V4\BasicApiController;
use App\Http\Resources\V4\InvoiceResource;
use App\Http\Resources\V4\InvoicesResource;
use App\Http\Services\V4\InvoiceService;
use App\Models\{BillAdd, Invoice};
use Illuminate\Http\Request;

class InvoiceController extends BasicApiController
{
    public function additions()
    {
        $rows = BillAdd::select('id', 'addition_name as name', 'check_addition as type', 'addition_value as value')->whereIn('add_role', [0, 1])->where('check_bill_type', '!=', 2)->get()->toArray();
        return $this->returnData($rows);
    }

    public function sales()
    {
        return $this->returnData(InvoicesResource::collection( $this->query( Invoice::SALES ) ));
    }

    public function backSales()
    {
        return $this->returnData(InvoicesResource::collection( $this->query( Invoice::BACK_SALES ) ));
    }

    public function purchases()
    {
        return $this->returnData(InvoicesResource::collection( $this->query( Invoice::PURCHASES ) ));
    }

    public function backPurchases()
    {
        return $this->returnData(InvoicesResource::collection( $this->query( Invoice::BACK_PURCHASES ) ));
    }

    public function query(int $type = 1)
    {
        return Invoice::select('id', 'bill_no', 'client_id', 'invoice_date', 'net_price', 'local_bill_no')->with('client')->where('invoice_type', $type)
                ->withCount('details')->with('client')->whereDate('invoice_date', request()->get('date', date('Y-m-d')))->get();
    }

    public function newBill(Request $request, InvoiceService $service)
    {
        $response = $service->handler($request->all(), Invoice::SALES);
        if ($response['status'] == 200) return $this->sendResponse(result: ['data' => $response['data']]);
        return $this->sendError($response['message'], $response['errors'] ?? []);
    }

    public function newBackBill(Request $request, InvoiceService $service)
    {
        $response = $service->handler($request->all(), Invoice::BACK_SALES);
        if ($response['status'] == 200) return $this->sendResponse(result: ['data' => $response['data']]);
        return $this->sendError($response['message']);
    }

    public function show($id)
    {
        session(['currency' => shop()->currency]);
        $bill = Invoice::select('id', 'bill_no', 'invoice_date', 'net_price', 'local_bill_no', 'client_id', 'payment', 'invoice_total_disc', 'invoice_total_disc_type', 'shop_id')
                                ->with('client', 'details', 'adds')->where('id', $id)->first();

        return $bill
                ? $this->sendResponse(result: ['data' => new InvoiceResource($bill)])
                : $this->sendError('This bill not found');
    }
}
