<?php

namespace App\Http\Controllers\V4\Rep;

use App\Http\Controllers\V4\GeneralApiController;
use App\Http\Requests\V4\VisitCaseRequest;
use App\Http\Resources\V4\VisitCaseResource;
use App\Http\Services\V4\NegativeVisitService;
use App\Models\VisitCase;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Schema;

class VisitCaseController extends GeneralApiController
{
    public function __construct()
    {
        $this->checkDatabase();
        parent::__construct(VisitCase::class, VisitCaseResource::class);
    }

    public function store(VisitCaseRequest $request)
    {
        Log::debug(json_encode($request->all()));
        $result = NegativeVisitService::create( $request->data );
        return $result['status']
                    ? $this->sendResponse('تم حفظ البيانات بنجاح', ['errors' => $result['errors']])
                    : $this->sendError('هناك خطأ ما');
    }

    public function query(?int $id = null): \Illuminate\Database\Eloquent\Builder
    {
        return $this->model::query()->select('id', 'case');
    }

    protected function checkDatabase()
    {
        if (!Schema::hasTable('visiting_cases')) {
            DB::statement("CREATE TABLE `visiting_cases` (
                `id` int UNSIGNED NOT NULL,
                `case` text CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci NOT NULL,
                `shop_id` int UNSIGNED NOT NULL,
                `created_at` timestamp NULL DEFAULT NULL,
                `updated_at` timestamp NULL DEFAULT NULL
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;");
            DB::statement("ALTER TABLE `visiting_cases` ADD PRIMARY KEY (`id`);");
            DB::statement("ALTER TABLE `visiting_cases` MODIFY `id` int UNSIGNED NOT NULL AUTO_INCREMENT, AUTO_INCREMENT=1;");
        }

        if (!Schema::hasTable('visits')) {
            DB::statement("CREATE TABLE `visits` (
                `id` int NOT NULL,
                `visit_date` date NOT NULL,
                `notes` text CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci,
                `client_id` int NOT NULL,
                `employee_id` int NOT NULL,
                `status` tinyint(1) NOT NULL DEFAULT '0',
                `shop_id` int NOT NULL,
                `user_id` int NOT NULL,
                `created_at` timestamp NULL DEFAULT NULL,
                `updated_at` timestamp NULL DEFAULT NULL
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb3");
            DB::statement("ALTER TABLE `visits` ADD PRIMARY KEY (`id`);");
            DB::statement("ALTER TABLE `visits` MODIFY `id` int UNSIGNED NOT NULL AUTO_INCREMENT, AUTO_INCREMENT=1;");
        }

        if (!Schema::hasTable('negative_visits')) {
            DB::statement("CREATE TABLE `negative_visits` (
                `id` int UNSIGNED NOT NULL,
                `notes` text CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci,
                `user_id` int UNSIGNED NOT NULL,
                `client_id` int UNSIGNED NOT NULL,
                `shop_id` int UNSIGNED NOT NULL,
                `visie_date` date DEFAULT NULL,
                `created_at` timestamp NULL DEFAULT NULL,
                `updated_at` timestamp NULL DEFAULT NULL,
                `reason_id` int UNSIGNED NOT NULL,
                `branch_id` int UNSIGNED DEFAULT NULL
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;");
            DB::statement("ALTER TABLE `negative_visits` ADD PRIMARY KEY (`id`);");
            DB::statement("ALTER TABLE `negative_visits` MODIFY `id` int UNSIGNED NOT NULL AUTO_INCREMENT, AUTO_INCREMENT=1;");
        }
    }
}
