<?php

namespace App\Http\Requests\V4;

use App\Models\Badrshop;
use App\Traits\HandleValidationError;
use Illuminate\Foundation\Http\FormRequest;

class ClientRequest extends FormRequest
{
    use HandleValidationError;

    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, mixed>
     */
    public function rules()
    {
        $setting = Badrshop::select('telephone_required', 'client_address_required')->first() ?? new Badrshop();

        $telephone_required = $setting->telephone_required ? 'required' : 'nullable';
        $client_address_required = $setting->client_address_required ? 'required' : 'nullable';

        return [
            'clients.*'                   => 'required|array',
            'clients.*.client_id'         => 'nullable',
            'clients.*.client_name'       => 'required',
            'clients.*.tele'              => "$telephone_required|numeric",
            'clients.*.mobile1'           => 'nullable|numeric',
            'clients.*.balance'           => 'nullable|numeric',
            'clients.*.client_tax_number' => 'nullable|string|max:191',
            'clients.*.lat'               => 'nullable|numeric',
            'clients.*.lon'               => 'nullable|numeric',
            'clients.*.address'           => "$client_address_required|string",
            'clients.*.addition_data'     => 'nullable|string',
            'clients.*.credit_limit'      => 'nullable|numeric|min:0',
            'clients.*.date_time'         => 'nullable|date_format:Y-m-d H:i:s',
            'clients.*.days'              => 'nullable|array',
            'clients.*.days.*'            => 'nullable',

        ];
    }

    public function singleRules(?int $client_id = null)
    {
        $setting = Badrshop::select('telephone_required', 'client_address_required')->first() ?? new Badrshop();

        $telephone_required = $setting->telephone_required ? 'required' : 'nullable';
        $client_address_required = $setting->client_address_required ? 'required' : 'nullable';

        return [
            'client_id'         => "nullable",
            'client_name'       => "required|string",
            // 'client_name'       => "required|unique:clients,client_name,$client_id",
            'tele'              => "$telephone_required|numeric|unique:clients,tele,$client_id",
            'mobile1'           => "nullable|numeric|unique:clients,mobile1,$client_id",
            'balance'           => "nullable|numeric",
            'client_tax_number' => "nullable|string|max:191",
            'lat'               => "nullable|numeric",
            'lon'               => "nullable|numeric",
            'address'           => "$client_address_required|string",
            'addition_data'     => "nullable|string",
            'credit_limit'      => "nullable|numeric|min:0",
            'date_time'         => "nullable|date_format:Y-m-d H:i:s",
            'days'              => "nullable|array",
            'days.*'            => "nullable",
        ];
    }

    public function attributes()
    {
        return [
            'client_name'        => trans('inputs.client_name'),
            'tele'               => trans('inputs.tele'),
            'mobile1'            => trans('inputs.mobile1'),
            'balance'            => trans('inputs.balance'),
            'client_tax_number'  => trans('inputs.client_tax_number'),
            'lat'                => trans('inputs.lat'),
            'lon'                => trans('inputs.lon'),
            'address'            => trans('inputs.address'),
            'addition_data'      => trans('inputs.addition_data'),
            'credit_limit'       => trans('inputs.credit_limit'),
            'date_zero'          => trans('inputs.date_zero'),
        ];
    }
}
