<?php

namespace App\Http\Resources\V4;

use App\Models\SalePoint;
use Illuminate\Http\Resources\Json\JsonResource;
use Illuminate\Support\Facades\DB;

class SettingsResource extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array|\Illuminate\Contracts\Support\Arrayable|\JsonSerializable
     */
    public function toArray($request)
    {
        $sale = SalePoint::select('money_point', 'store_id')->with('store')->whereId(salePointId())->first();
        return [
            'version'              => env('APP_VERSION', '3.1'),
            'multi_price'          => $this->multi_price,
            'select_client'        => $this->select_client,
            'check_lowest_price'   => $this->check_lowest_price,
            'tracking'             => $this->tracking,
            'bill_in_location'     => $this->make_bill,
            'logo'                 => $this->logo_path,
            'generate_qr'          => $this->generate_qr ?? 0,
            'company_ratio'        => $this->company_ratio,
            'sale_details'         => $this->sale_details,
            'bill_adds'            => $this->bill_adds,
            'sale_details_safe'    => $this->sale_details_safe,
            'allow_lines'          => $this->allow_lines,
            'point_balance'        => $sale?->money_point,
            'store_name'           => $sale?->store?->name,
            'can_edit_client_days' => authUser()->can_edit_client_days ?? 0,
            'show_pay_price'       => authUser()->show_pay_price ?? 0,
            'sale_cash'            => authUser()->sale_cash ?? 0,
            'sale_discount'        => authUser()->sale_discount ?? 0,
            'edit_sale_price'      => authUser()->sale_price ?? 0,
            'return_sales_insert'  => authUser()->return_sales_insert ?? 0,
            'telephone_required'   => $this->telephone_required ?? 0,
            'spending'             => authUser()->spending,
            'add_spend'            => authUser()->add_spend,
            'vat_number'           => DB::table('bill_setting')->select('vat_number')->where('shop_id', shopId())->first()->vat_number ?? null,
            'client_address_required' => $this->client_address_required ?? 0,
            'activate_maintenance_mode' => $this->activate_maintenance_mode
        ];
    }
}
