<?php

namespace App\Http\Services\V4;

class Calculator
{
    public float $quantity = 0;
    public float $price = 0;
    public float $total = 0;
    public float $net_price = 0;
    public float $payment = 0;
    public float $rest = 0;
    public float $water = 0;
    public float $protein = 0;
    public float $fats = 0;
    public float $water_discount = 0;
    public float $net_quantity = 0;
    public float $protein_bonus = 0;
    public float $fats_bonus = 0;

    public function calculate()
    {
        $this->calculateWaterDiscount();
        $this->calculateNetQuantity();
        $this->calculateFatsBonus();
        $this->calculateProteinBonus();
        $this->calculateTotalPrice();
        $this->calculateEndPrice();
        $this->calculateRest();
    }

    public function setQuantity(float $quantity): Calculator
    {
        $this->quantity = $quantity;
        return $this;
    }

    public function setWater(float $water): Calculator
    {
        $this->water = $water;
        return $this;
    }

    public function setProtein(float $protein): Calculator
    {
        $this->protein = $protein;
        return $this;
    }

    public function setFats(float $fats): Calculator
    {
        $this->fats = $fats;
        return $this;
    }

    public function setPrice($price): Calculator
    {
        $this->price = $price;
        return $this;
    }

    public function setPayment(?float $payment = 0): Calculator
    {
        $this->payment = $payment;
        return $this;
    }

    public function calculateWaterDiscount(): Calculator
    {
        $this->water_discount = $this->quantity * ($this->water / 100);
        return $this;
    }

    public function calculateNetQuantity(): Calculator
    {
        $this->net_quantity = $this->quantity - $this->water_discount;
        return $this;
    }

    public function calculateProteinBonus(): Calculator
    {
        $this->protein_bonus = $this->calculateBonus($this->protein, ['min' => 1.1, 'max' => 3.5], ['min' => 3.00, 'max' => 3.10], 0.03);
        return $this;
    }

    public function calculateFatsBonus(): Calculator
    {
        $this->fats_bonus = $this->calculateBonus($this->fats, ['min' => 2.1, 'max' => 4.20], ['min' => 3.50, 'max' => 3.70], 0.05);
        return $this;
    }

    public function calculateTotalPrice(): Calculator
    {
        $this->total = $this->fats_bonus + $this->protein_bonus + $this->price;
        return $this;
    }

    public function calculateEndPrice(): Calculator
    {
        $this->net_price = $this->total * $this->net_quantity;
        return $this;
    }

    public function calculateRest(): Calculator
    {
        $this->rest = $this->net_price - $this->payment;
        return $this;
    }

    protected function calculateBonus(float $percentage, array $maxRang, array $zeroRange, float $bonus_value): float
    {
        if ($percentage > $maxRang['max']) $percentage = $maxRang['max'];
        if ($percentage < $maxRang['min']) $percentage = $maxRang['min'];

        $percentage = number_format($percentage, 2);
        $result = 0;

        if ( $percentage >= $zeroRange['min'] && $percentage <= $zeroRange['max'] ) {
            $result = 0;
        } else if ( $percentage > $zeroRange['max'] ) {
            $result = $percentage - $zeroRange['max'];
        } else if ( $percentage < $zeroRange['min'] ) {
            $result = $percentage - $zeroRange['min'];
        }

        $bonus = $result * $bonus_value * 10;
        return number_format($bonus, 2);
    }
}
