<?php

namespace App\Http\Services\V4;

use App\Http\Requests\V4\ClientRequest;
use App\Http\Resources\V4\ClientsResource;
use App\Models\{Client, ClientTransaction, LineClient};
use Illuminate\Support\Facades\{DB, Validator};
use Exception;
use Illuminate\Support\Arr;
use PDOException;

class ClientService
{
    public Client $client;
    public $row;

    public function create(array $request, bool $add_lines)
    {
        $rows = [];
        $errors = [];
        DB::beginTransaction();
            foreach ($request['clients'] as $index => $row) {
                try{
                    $validator = Validator::make($row, (new ClientRequest())->singleRules( ($row['client_id'] ?? null) ));
                    if ($validator->fails()) {
                        $errors [] = Arr::flatten($validator->errors()->toArray());
                        logger( "validator : " . json_encode($errors) );
                        continue;
                    }
                    $this->row = $validator->validated();
                    $this->row['date_time'] = $this->row['date_time'] ?? now();
                    $this->row['date_zero'] = $this->row['date_time'];

                    $this->createClient()->createTransaction()->createLine($add_lines);

                    $this->client->local_id = $row['local_id'] ?? null;
                    $rows[] = $this->client;
                } catch(Exception $e) {
                    if ($e instanceof PDOException) {
                        $client = Client::where('tele', $this->row['tele'])->first();
                        $client->local_id = $row['local_id'] ?? null;
                        $rows[] = $client;
                    }
                    logger( "Exception : " . json_encode($e->getMessage()) );
                    continue;
                }
            }
        DB::commit();

        return ['rows' => ClientsResource::collection($rows), 'errors' => $errors];
    }

    protected function createClient()
    {
        $this->client = Client::updateOrCreate(['id' => $this->row['client_id'] ?? null], $this->row);

        return $this;
    }

    protected function createTransaction()
    {
        ClientTransaction::create([
                'date_time'     => $this->row['date_time'],
                'pay_day'       => now(),
                'type'          => 4,
                'client_id'     => $this->client->id,
                'amount'        => $this->client->balance,
                'balance'       => $this->client->balance,
                'user_id'       => $this->client->add_user,
                'bill_id'       => 0,
                'sale_back_id'  => 0,
                'supplier_id'   => 0,
                'problem_id'    => 0,
                'effect'        => 0,
                'salary_month'  => 0,
                'OutTransactionID'=> 0,
                'spend_id'      => 0,
            ]);

        return $this;
    }

    protected function createLine(bool $add_lines)
    {
        if (! $add_lines) return $this;
        $data = [];
        if ($this->row['days']) {
            foreach ($this->row['days'] as $day) {
                $data[] = [
                    'line_id'   => authUser()->line?->id,
                    'client_id' => $this->client->id,
                    "$day"      => true
                ];
            }

            if ( count($data) && authUser()->line ) LineClient::insert($data);
        }

        return $this;
    }
}
