<?php

namespace App\Models;

use App\Models\Scopes\HasShopScope;
use Illuminate\Database\Eloquent\Casts\Attribute;
use Illuminate\Database\Eloquent\Model;

class Invoice extends Model
{
    protected $table = 'invoices_';
    protected $guarded = ['id'];
    public $timestamps = false;

    const PURCHASES = 0;
    const SALES = 1;
    const BACK_PURCHASES = 2;
    const BACK_SALES = 3;

    protected function billTotal(): Attribute
    {
        return Attribute::make(
            get: fn ($value) => format_price($value ?? 0),
        );
    }

    protected function netPrice(): Attribute
    {
        return Attribute::make(
            get: fn ($value) => format_price($value ?? 0),
        );
    }

    protected function payment(): Attribute
    {
        return Attribute::make(
            get: fn ($value) => format_price($value ?? 0),
        );
    }

    protected function billPayment(): Attribute
    {
        return Attribute::make(
            get: fn ($value) => format_price($value ?? 0),
        );
    }

    protected function rest(): Attribute
    {
        return Attribute::make(
            get: fn ($value) => format_price($value ?? 0),
        );
    }

    protected function balance(): Attribute
    {
        return Attribute::make(
            get: fn ($value) => format_price($value ?? 0),
        );
    }

    protected function discount(): Attribute
    {
        return Attribute::make(
            get: fn ($value) => $this->invoice_total_disc > 0 ? ($this->invoice_total_disc . ' ' . ($this->invoice_total_disc_type ? session()->get('currency') : ' % ')) : '0',
        );
    }

    public function details()
    {
        return $this->hasMany(InvoiceDetails::class, 'invoice_id', 'id')->select('id', 'invoice_id', 'item_id', 'unit_id', 'quantity', 'quantity_after', 'sale_price')->with('item', 'unit');
    }

    public function client()
    {
        return $this->belongsTo(Client::class, 'client_id', 'id')->select('id', 'client_name as name')->withDefault(['id' => '', 'name' => '']);
    }

    public function adds()
    {
        return $this->hasMany(BillAddHistory::class, 'bill_id', 'id')->select('bill_id', 'addition_value', 'bill_add', 'add_type', 'addition_id');
    }

    public static function generateUniqueRow(object $invoice)
    {
        // client_id-invoice_date-net_price-shop_id-add_user-invoice_type
        return "{$invoice->client_id}-{$invoice->invoice_date}-{$invoice->net_price}-{$invoice->shop_id}-{$invoice->add_user}-{$invoice->invoice_type}";
    }

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope(new HasShopScope);

        static::creating(function ($model) {
            $model->shop_id                 = shopId();
            $model->transaction_date        = now();
            $model->shop_id                 = shopId();
            $model->bill_no                 = self::query()->max('bill_no') + 1;
            $model->add_user                = authUser()->id ?? 0;
            $model->invoice_total_disc_type = 1;
            $model->sales_man               = 0;
            $model->bill_service_id         = 0;
            $model->bill_service_value      = 0;
            $model->delivery_option_id      = 0;
            $model->delivery_period_id      = 0;
            $model->delivery_man_id         = 0;
            $model->submissionId            = '';
            $model->uuid                    = '';
            $model->longId                  = '';
            $model->internalId              = '';
            $model->hashKey                 = '';
            $model->unique_row              = self::generateUniqueRow($model);
        });
    }
}
