<?php

namespace App\Models;

use App\Models\Scopes\HasShopScope;
use Illuminate\Database\Eloquent\Casts\Attribute;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\Hash;
use Laravel\Sanctum\HasApiTokens;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'name',
        'email',
        'password',
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    protected function password(): Attribute
    {
        return Attribute::make(
            set: fn ($value) => Hash::make($value),
        );
    }

    public function shop()
    {
        return $this->belongsTo(Badrshop::class, 'shop_id', 'serial_id')
                    ->select('serial_id', 'run_date', 'generate_qr', 'online', 'country', 'shop_name', 'decimal_num_price', 'decimal_num_quant', 'sale_details', 'currency', 'allow_lines', 'bill_adds', 'zone')
                    ->withDefault(['decimal_num_price' => 2, 'decimal_num_price' => 2, 'shop_name' => '', 'run_date' => now(), 'online' => false]);
    }

    public function salePoint()
    {
        return $this->belongsTo(SalePoint::class, 'sale_point', 'id')->with('store');
    }

    public function line()
    {
        return $this->hasOne(Line::class, 'representative_id', 'id');
    }

    public function groups()
    {
        return $this->hasMany(UserGroup::class, 'user_id', 'id');
    }

    protected static function boot()
    {
        parent::boot();

        if (authUser()) {
            static::addGlobalScope(new HasShopScope);
        }
    }
}
